
public:
	
	/** Set the number of AC coefficients.
	  * @param y Number of Y AC coefficients.
	  * @param c Number of Cb/Cr AC coefficients.
	  */
	virtual void SetNrCoefficients(unsigned* y, unsigned* c, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	/** Get number of Y coefficents (DC+AC). */
	virtual unsigned GetNrYCoeffs();

	/** Get number of Cb/Cr AC coefficents (DC+AC). */
	virtual unsigned GetNrCCoeffs();

	/** Set Y coefficents.
	  * @param coeffs The number of coefficients stored in this array must equal the number of DC
	  *    + AC coefficients. The coefficients must be ordered by ascending frequency. */
	virtual void SetY(unsigned* coeffs, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	/** Get Y coefficents.
	  * param The size of this array must equal the number of DC
	  *    + AC coefficients. The coefficients are ordered by ascending frequency. */
	virtual void GetY(unsigned* coeffs);

	/** Set Cb coefficents.
	  * @param coeffs The number of coefficients stored in this array must equal the number of DC
	  *    + AC coefficients. The coefficients must be ordered by ascending frequency. */
	virtual void SetCb(unsigned* coeffs, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	/** Get Cb coefficents.
	  * param The size of this array must equal the number of DC
	  *    + AC coefficients. The coefficients are ordered by ascending frequency. */
	virtual void GetCb(unsigned* coeffs);

	/** Set Cr coefficents.
	  * @param coeffs The number of coefficients stored in this array must equal the number of DC
	  *    + AC coefficients. The coefficients must be ordered by ascending frequency. */
	virtual void SetCr(unsigned* coeffs, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	/** Get Cr coefficents.
	  * param The size of this array must equal the number of DC
	  *    + AC coefficients. The coefficients are ordered by ascending frequency. */
	virtual void GetCr(unsigned* coeffs);


private:
	/** number of coefficients to be used for MPEG-7 representation
	  * @param nrCoeffs nr of defined coefficients
	  * @return one of the predefined numbers of coefficients
	  */
	unsigned _NrCoeffsToUse(unsigned nrCoeffs);

	/** init the y coefficents list */
	void _CreateCoeffListsY(unsigned y, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	/** init the Cb/Cr coefficients lists */
	void _CreateCoeffListsC(unsigned c, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);






