/*****************************************************************************/
/* Copyright (C) 2003-2006, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/* This software may only be used under the terms of the JRS MPEG-7 Library  */
/* license, available at                                                     */
/* http://iis.joanneum.at/MPEG-7/JRS-MPEG-7-Lib-License.pdf                  */
/*****************************************************************************/


#ifndef MP7JRSEXCEPTION_H
#define MP7JRSEXCEPTION_H

#include <xercesc/util/XMLString.hpp>
#include "Mp7JrsDefines.h"

typedef enum {
	MP7JRS_WARNING,
	MP7JRS_ERROR,
	MP7JRS_FATAL_ERROR
} Mp7JrsExSeverity;

typedef enum {
	MP7JRS_AUTODELETE, // Message created on stack, nothing to do
	MP7JRS_NOAUTODELETE, // Message created on our heap (NOT Xerces heap), we have to destroy it.
} Mp7JrsMsgMemory;

/** Primary exception class. */
class MP7JRS_EXPORT Mp7JrsException 
{
public:
	/**
	 * This is version 2.0
	 */
	Mp7JrsException(Mp7JrsExSeverity sev, const XMLCh* msg, Mp7JrsMsgMemory mode = MP7JRS_AUTODELETE) 
		: _severity(sev) { 
		_msg = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(msg); // Xerces heap
		if(mode == MP7JRS_NOAUTODELETE && msg){
			XMLCh * tmp = const_cast<XMLCh*>(msg);
			delete [] tmp; // Our heap
		}		
	}

	/**
	 * This is version 2.0
	 */
	Mp7JrsException(Mp7JrsExSeverity sev, const char* msg, Mp7JrsMsgMemory mode = MP7JRS_AUTODELETE) 
		: _severity(sev) { 
		_msg = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::replicate(msg);  // Xerces heap
		if(mode == MP7JRS_NOAUTODELETE && msg){
			char * tmp = const_cast<char*>(msg);
			delete [] tmp; // Our heap
		}
	}

	// Copy ctor, otherwise we leave a memory leak!
	Mp7JrsException(const Mp7JrsException & other)
	{
		_severity = other._severity;
		_msg = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::replicate(other._msg);  // Xerces heap
	}
	
	virtual ~Mp7JrsException()
	{
		XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&_msg); // Xerces heap, _msg is reset to NULL
	}
	
	/** Get severity identifier */
	Mp7JrsExSeverity GetSeverity() { return _severity; };

	/** Get the severity as string. */
	const char* GetSeverityText() {
		switch (_severity) {
		case MP7JRS_WARNING: return "Warning"; 
		case MP7JRS_ERROR: return "Error";
		case MP7JRS_FATAL_ERROR: return "Fatal error";
		default: return "";
		};
	}

	/** Get a descriptive error message. */
	const char* GetMessage() { return _msg; }

private:
	Mp7JrsExSeverity	_severity;
	char*				_msg;
};

#endif
