/*****************************************************************************/
/* Copyright (C) 2003-2006, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/* This software may only be used under the terms of the JRS MPEG-7 Library  */
/* license, available at                                                     */
/* http://iis.joanneum.at/MPEG-7/JRS-MPEG-7-Lib-License.pdf                  */
/*****************************************************************************/

// ClassType_H.template
#ifndef _1207MP7JRSMEDIAQUALITYTYPE_H
#define _1207MP7JRSMEDIAQUALITYTYPE_H

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOM.hpp>

#include "Mp7JrsDefines.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsMediaQualityTypeFactory.h"
#include "Mp7JrsNode.h"
#include "Mp7JrsConvert.h"


// no includefile for extension defined 
// file Mp7JrsMediaQualityType_ExtInclude.h


#include "Mp7JrsDType.h"
#include "Mp7JrsMpeg7BaseType.h"
class IMp7JrsAgentType;
class IMp7JrsDType;
class IMp7JrsMediaQualityType;
class IMp7JrsMediaQualityType_PerceptibleDefects_LocalType;
class IMp7JrsMediaQualityType_QualityRating_CollectionType;
class IMp7JrsMediaQualityType_RatingInformationLocator_CollectionType;
typedef Mp7JrsPtr< IMp7JrsAgentType > Mp7JrsAgentPtr;
typedef Mp7JrsPtr< IMp7JrsDType > Mp7JrsDPtr;
typedef Mp7JrsPtr< IMp7JrsMediaQualityType_PerceptibleDefects_LocalType > Mp7JrsMediaQualityType_PerceptibleDefects_LocalPtr;
typedef Mp7JrsPtr< IMp7JrsMediaQualityType_QualityRating_CollectionType > Mp7JrsMediaQualityType_QualityRating_CollectionPtr;
typedef Mp7JrsPtr< IMp7JrsMediaQualityType_RatingInformationLocator_CollectionType > Mp7JrsMediaQualityType_RatingInformationLocator_CollectionPtr;
typedef Mp7JrsPtr< IMp7JrsMediaQualityType> Mp7JrsMediaQualityPtr;

/** 
 * Generated interface IMp7JrsMediaQualityType for class Mp7JrsMediaQualityType<br>
 * Located at: mds-2001.xsd, line 2400<br>
 * Classified: Class<br>
 * Derived from: DType (Class)<br>
 */
class MP7JRS_EXPORT IMp7JrsMediaQualityType 
 :
        public IMp7JrsDType
{
public:
    // TODO: make these protected?
    IMp7JrsMediaQualityType();
    virtual ~IMp7JrsMediaQualityType();
        /** @name Sequence

         */
        //@{
    /**
     * Get QualityRating element.
     * @return Mp7JrsMediaQualityType_QualityRating_CollectionPtr
     */
    virtual Mp7JrsMediaQualityType_QualityRating_CollectionPtr GetQualityRating() const = 0;

    /**
     * Set QualityRating element.
     * @param item const Mp7JrsMediaQualityType_QualityRating_CollectionPtr &
     */
    // Mandatory collection
    virtual void SetQualityRating(const Mp7JrsMediaQualityType_QualityRating_CollectionPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

    /**
     * Get RatingSource element.
     * @return Mp7JrsPtr< Mp7JrsNode >
     * Possible types (within this library):
     * <ul>
     * <li>IMp7JrsOrganizationPtr</li>
     * <li>IMp7JrsPersonGroupPtr</li>
     * <li>IMp7JrsPersonPtr</li>
     * </ul>
     */
    virtual Mp7JrsPtr< Mp7JrsNode > GetRatingSource() const = 0;

    virtual bool IsValidRatingSource() const = 0;

    /**
     * Set RatingSource element.
     * @param item const Mp7JrsPtr< Mp7JrsNode > &
     * Valid types for item:
     * <ul>
     * <li>IMp7JrsOrganizationPtr</li>
     * <li>IMp7JrsPersonGroupPtr</li>
     * <li>IMp7JrsPersonPtr</li>
     * </ul>
     */
    virtual void SetRatingSource(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate RatingSource element.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidateRatingSource() = 0;

    /**
     * Get RatingInformationLocator element.
     * @return Mp7JrsMediaQualityType_RatingInformationLocator_CollectionPtr
     */
    virtual Mp7JrsMediaQualityType_RatingInformationLocator_CollectionPtr GetRatingInformationLocator() const = 0;

    /**
     * Set RatingInformationLocator element.
     * @param item const Mp7JrsMediaQualityType_RatingInformationLocator_CollectionPtr &
     */
    // Mandatory collection
    virtual void SetRatingInformationLocator(const Mp7JrsMediaQualityType_RatingInformationLocator_CollectionPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

    /**
     * Get PerceptibleDefects element.
     * @return Mp7JrsMediaQualityType_PerceptibleDefects_LocalPtr
     */
    virtual Mp7JrsMediaQualityType_PerceptibleDefects_LocalPtr GetPerceptibleDefects() const = 0;

    virtual bool IsValidPerceptibleDefects() const = 0;

    /**
     * Set PerceptibleDefects element.
     * @param item const Mp7JrsMediaQualityType_PerceptibleDefects_LocalPtr &
     */
    virtual void SetPerceptibleDefects(const Mp7JrsMediaQualityType_PerceptibleDefects_LocalPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate PerceptibleDefects element.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidatePerceptibleDefects() = 0;

        //@}
    /**
     * Gets or creates Mp7JrsNodePtr child elements of MediaQualityType.
     * Currently this type contains 4 child elements of type "Mp7JrsNodePtr".

     * <ul>
     * <li>QualityRating</li>
     * <li>RatingSource</li>
     * <li>RatingInformationLocator</li>
     * <li>PerceptibleDefects</li>
     * </ul>
     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     * \par<code>A</code> 
     * - Select all child elements named <code>A</code>.
     * - In case of no element, create a new one.
     * - For collections: all subsequent XPath expressions are applied on all collection items (e.g. <code>A/B</code> query or create, childnodes <code>B</code> of all <code>A</code>s). 
     * \par<code>A[i]</code>
     * - Select the <code>i</code>th child element of all <code>A</code>.
     * - Please note, that XPath indexes start at 1.
     * - If there is no such <code>i</code>th element, try to create a new one at index <code>i</code>.
     * - Single elements require <code>i</code> to be 1 otherwise an exception is thrown.
     * - For collections, <code>i</code> must not exceed the number of existing items +1 (appending is allowed, filling gaps not).
     * - If the index is too high, a <code>MissingElementException</code> is thrown, and you can extract the last valid index. (Example: If a collection contains 2 elements and you are requesting the 4th element).
     * 
     * \par <code>A[\@xsi:type="T"]</code>
     *     - Select all child elements named <code>A</code> of type <code>T</code>.
     *     - Currently only typenames without namespace are recognised.
     *     - If there is no such element, create and append a new one.
     * \par <code>A[i][\@xsi:type="T"]</code>
     *     - Select the <code>i</code>th child element named <code>A</code>, which must be of type <code>T</code>.
     *     - If there is no such element at index <code>i</code>, a new one is created and inserted (appended).
     * \par <code>A[\@xsi:type="T"][i]</code>
     *     - Out of all child elements named <code>A</code> of type <code>T</code>, take the <code>i</code>th one.
     *     - If there is no such element at index <code>i</code>, create and append one.
     * 
     * \par <code>A[last()]</code>
     *     - Select the last child element named <code>A</code>.
     *     - In case of no element, create a new one.
     * \par <code>A[last()][\@xsi:type="T"]</code>
     *     - Select the last child element named <code>A</code>, which must be of type <code>T</code>.
     *     - For collections: in case of no such type, create and append a new one.
     *     - For single elements: if an element with different type exists, return the empty list.
     * 
     * \par <code>A[\@xsi:type="T"][last()]</code>
     *     - Out of all child elements named <code>A</code> of type <code>T</code>, take the last one.
     *     - For collections: in case of no such type, create and append a new one.
     *     - For single elements: if an element with different type exists, return the empty list.
     * 
     * \par <code>A[position() \> last()]</code>
     *     - Force creation of a child element named <code>A</code> with its default type.
     *     - For collections: create and append the default element.
     *     - For single elements: if any element exists, an exception is thrown.
     *
     * \par <code>A[position() \> last()][\@xsi:type="T"]</code>
     *     - Force creation of a child element named <code>A</code> of type <code>T</code>.
     *     - For collections: create and append the element.
     *     - For single elements: if any element exists, an exception is thrown.
     *
     * \par <code>A[\@xsi:type="T"][position() \> last()]</code>
     *     - Force creation of a child element named <code>A</code> of type <code>T</code>.
     *     - Behaves the same way as the previous expression.
     * 
     * 
     * Supported XPath expressions for GetFromXPath<br>
     * The same expressions like GetOrCreateFromXPath plus
     * \par <code>A[position() \< i] </code>
     *     - Select all child elements named <code>A</code> at indexes lower than <code>i</code>.
     *     - <code>i</code> must be greater than -1, otherwise an exception is thrown.
     * \par <code>A[position() \> i] </code>
     *     - Select all child elements named <code>A</code> at indexes greater than <code>i</code>.
     *     - <code>i</code> must be greater than -1, otherwise an exception is thrown.
     * \par <code>A[position() \> i | position() \< j] </code>
     *     - Select all child elements named <code>A</code> at indexes greater than <code>i</code> and lower than <code>j</code>.
     *     - <code>i</code> and <code>j</code> must be greater than -1, otherwise an exception is thrown.
     *     - If <code>i</code> is greater than <code>j</code> then an empty list is returned.
     *
     * These three expressions can be combined with the <code>\@xsi:type</code> attribute to get a more restricted query.
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL) = 0;

    
// no includefile for extension defined 
// file Mp7JrsMediaQualityType_ExtMethodDef.h


// no includefile for extension defined 
// file Mp7JrsMediaQualityType_ExtPropDef.h

};




/*
 * Generated class Mp7JrsMediaQualityType<br>
 * Located at: mds-2001.xsd, line 2400<br>
 * Classified: Class<br>
 * Derived from: DType (Class)<br>
 * Defined in mds-2001.xsd, line 2400.<br>
 */
class MP7JRS_EXPORT Mp7JrsMediaQualityType :
        public IMp7JrsMediaQualityType
{
    friend class Mp7JrsMediaQualityTypeFactory; // constructs objects

public:
    /*
     * Retrieves the base class of Mp7JrsMediaQualityType
     
     * @return Mp7JrsPtr< Mp7JrsDType >
     */
    virtual Mp7JrsPtr< Mp7JrsDType > GetBase() const;
        /* @name Sequence

         */
        //@{
    /*
     * Get QualityRating element.
     * @return Mp7JrsMediaQualityType_QualityRating_CollectionPtr
     */
    virtual Mp7JrsMediaQualityType_QualityRating_CollectionPtr GetQualityRating() const;

    /*
     * Set QualityRating element.
     * @param item const Mp7JrsMediaQualityType_QualityRating_CollectionPtr &
     */
    // Mandatory collection
    virtual void SetQualityRating(const Mp7JrsMediaQualityType_QualityRating_CollectionPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

    /*
     * Get RatingSource element.
     * @return Mp7JrsPtr< Mp7JrsNode >
     * Possible types (within this library):
     * <ul>
     * <li>IMp7JrsOrganizationPtr</li>
     * <li>IMp7JrsPersonGroupPtr</li>
     * <li>IMp7JrsPersonPtr</li>
     * </ul>
     */
    virtual Mp7JrsPtr< Mp7JrsNode > GetRatingSource() const;

    virtual bool IsValidRatingSource() const;

    /*
     * Set RatingSource element.
     * @param item const Mp7JrsPtr< Mp7JrsNode > &
     * Valid types for item:
     * <ul>
     * <li>IMp7JrsOrganizationPtr</li>
     * <li>IMp7JrsPersonGroupPtr</li>
     * <li>IMp7JrsPersonPtr</li>
     * </ul>
     */
    virtual void SetRatingSource(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate RatingSource element.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidateRatingSource();

    /*
     * Get RatingInformationLocator element.
     * @return Mp7JrsMediaQualityType_RatingInformationLocator_CollectionPtr
     */
    virtual Mp7JrsMediaQualityType_RatingInformationLocator_CollectionPtr GetRatingInformationLocator() const;

    /*
     * Set RatingInformationLocator element.
     * @param item const Mp7JrsMediaQualityType_RatingInformationLocator_CollectionPtr &
     */
    // Mandatory collection
    virtual void SetRatingInformationLocator(const Mp7JrsMediaQualityType_RatingInformationLocator_CollectionPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

    /*
     * Get PerceptibleDefects element.
     * @return Mp7JrsMediaQualityType_PerceptibleDefects_LocalPtr
     */
    virtual Mp7JrsMediaQualityType_PerceptibleDefects_LocalPtr GetPerceptibleDefects() const;

    virtual bool IsValidPerceptibleDefects() const;

    /*
     * Set PerceptibleDefects element.
     * @param item const Mp7JrsMediaQualityType_PerceptibleDefects_LocalPtr &
     */
    virtual void SetPerceptibleDefects(const Mp7JrsMediaQualityType_PerceptibleDefects_LocalPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate PerceptibleDefects element.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidatePerceptibleDefects();

        //@}
    /*
     * Gets or creates Mp7JrsNodePtr child elements of MediaQualityType.
     * Currently this type contains 4 child elements of type "Mp7JrsNodePtr".

     * <ul>
     * <li>QualityRating</li>
     * <li>RatingSource</li>
     * <li>RatingInformationLocator</li>
     * <li>PerceptibleDefects</li>
     * </ul>
     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     * \par<code>A</code> 
     * - Select all child elements named <code>A</code>.
     * - In case of no element, create a new one.
     * - For collections: all subsequent XPath expressions are applied on all collection items (e.g. <code>A/B</code> query or create, childnodes <code>B</code> of all <code>A</code>s). 
     * \par<code>A[i]</code>
     * - Select the <code>i</code>th child element of all <code>A</code>.
     * - Please note, that XPath indexes start at 1.
     * - If there is no such <code>i</code>th element, try to create a new one at index <code>i</code>.
     * - Single elements require <code>i</code> to be 1 otherwise an exception is thrown.
     * - For collections, <code>i</code> must not exceed the number of existing items +1 (appending is allowed, filling gaps not).
     * - If the index is too high, a <code>MissingElementException</code> is thrown, and you can extract the last valid index. (Example: If a collection contains 2 elements and you are requesting the 4th element).
     * 
     * \par <code>A[\@xsi:type="T"]</code>
     *     - Select all child elements named <code>A</code> of type <code>T</code>.
     *     - Currently only typenames without namespace are recognised.
     *     - If there is no such element, create and append a new one.
     * \par <code>A[i][\@xsi:type="T"]</code>
     *     - Select the <code>i</code>th child element named <code>A</code>, which must be of type <code>T</code>.
     *     - If there is no such element at index <code>i</code>, a new one is created and inserted (appended).
     * \par <code>A[\@xsi:type="T"][i]</code>
     *     - Out of all child elements named <code>A</code> of type <code>T</code>, take the <code>i</code>th one.
     *     - If there is no such element at index <code>i</code>, create and append one.
     * 
     * \par <code>A[last()]</code>
     *     - Select the last child element named <code>A</code>.
     *     - In case of no element, create a new one.
     * \par <code>A[last()][\@xsi:type="T"]</code>
     *     - Select the last child element named <code>A</code>, which must be of type <code>T</code>.
     *     - For collections: in case of no such type, create and append a new one.
     *     - For single elements: if an element with different type exists, return the empty list.
     * 
     * \par <code>A[\@xsi:type="T"][last()]</code>
     *     - Out of all child elements named <code>A</code> of type <code>T</code>, take the last one.
     *     - For collections: in case of no such type, create and append a new one.
     *     - For single elements: if an element with different type exists, return the empty list.
     * 
     * \par <code>A[position() \> last()]</code>
     *     - Force creation of a child element named <code>A</code> with its default type.
     *     - For collections: create and append the default element.
     *     - For single elements: if any element exists, an exception is thrown.
     *
     * \par <code>A[position() \> last()][\@xsi:type="T"]</code>
     *     - Force creation of a child element named <code>A</code> of type <code>T</code>.
     *     - For collections: create and append the element.
     *     - For single elements: if any element exists, an exception is thrown.
     *
     * \par <code>A[\@xsi:type="T"][position() \> last()]</code>
     *     - Force creation of a child element named <code>A</code> of type <code>T</code>.
     *     - Behaves the same way as the previous expression.
     * 
     * 
     * Supported XPath expressions for GetFromXPath<br>
     * The same expressions like GetOrCreateFromXPath plus
     * \par <code>A[position() \< i] </code>
     *     - Select all child elements named <code>A</code> at indexes lower than <code>i</code>.
     *     - <code>i</code> must be greater than -1, otherwise an exception is thrown.
     * \par <code>A[position() \> i] </code>
     *     - Select all child elements named <code>A</code> at indexes greater than <code>i</code>.
     *     - <code>i</code> must be greater than -1, otherwise an exception is thrown.
     * \par <code>A[position() \> i | position() \< j] </code>
     *     - Select all child elements named <code>A</code> at indexes greater than <code>i</code> and lower than <code>j</code>.
     *     - <code>i</code> and <code>j</code> must be greater than -1, otherwise an exception is thrown.
     *     - If <code>i</code> is greater than <code>j</code> then an empty list is returned.
     *
     * These three expressions can be combined with the <code>\@xsi:type</code> attribute to get a more restricted query.
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL);

    

    /* Get the class name.
     * @return The name of this class.
     */
    virtual XMLCh * ToText() const;

    /* Parse this type.
     * @param txt buffer to parse from.
     * @return true if parsing was successful, false otherwise.
     */
    virtual bool Parse(const XMLCh * const txt);

    /* Serialialize this object to a DOM tree.
     * @param doc DOM document.
     * @param parent Parent node in DOM tree.
     */
    virtual void Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem = NULL);

    /* Deserialize this object from a DOM tree.
     * @param doc DOM document.
     * @param parent Parent element in DOM tree.
     * @param current Current element in DOM tree.
     */
    virtual bool Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current);

    /* Creates and inserts a Childnode of the specified Type
     * @param type The tag type of the child to be created (tag name).
     * @param xsitype The xsi type of the child to be created if available.
     * @return the created child with the correct dynamic type for the specified type.
     */
    virtual Mp7JrsNodePtr CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype);
         
// no includefile for extension defined 
// file Mp7JrsMediaQualityType_ExtMyMethodDef.h


public:

    virtual ~Mp7JrsMediaQualityType();

protected:
    Mp7JrsMediaQualityType();
    virtual void DeepCopy(const Mp7JrsNodePtr &original);

    virtual Mp7JrsNodeEnum GetAllChildElements() const;

    virtual Mp7JrsNodePtr GetBaseRoot();
    virtual const Mp7JrsNodePtr GetBaseRoot() const;

private:

    // Base Class
    Mp7JrsPtr< Mp7JrsDType > m_Base;

    Mp7JrsMediaQualityType_QualityRating_CollectionPtr m_QualityRating;
    Mp7JrsPtr< Mp7JrsNode > m_RatingSource;
    bool m_RatingSource_Exist; // For optional elements 
    Mp7JrsMediaQualityType_RatingInformationLocator_CollectionPtr m_RatingInformationLocator;
    Mp7JrsMediaQualityType_PerceptibleDefects_LocalPtr m_PerceptibleDefects;
    bool m_PerceptibleDefects_Exist; // For optional elements 

// no includefile for extension defined 
// file Mp7JrsMediaQualityType_ExtMyPropDef.h


public:
    virtual void Init(); // Called by inheritees to recursively update m_myself
protected:
    void Cleanup();

};

#endif // _1207MP7JRSMEDIAQUALITYTYPE_H

