/*****************************************************************************/
/* Copyright (C) 2003-2006, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/* This software may only be used under the terms of the JRS MPEG-7 Library  */
/* license, available at                                                     */
/* http://iis.joanneum.at/MPEG-7/JRS-MPEG-7-Lib-License.pdf                  */
/*****************************************************************************/

// ClassType_H.template
#ifndef _1225MP7JRSMEDIARELTIMEPOINTTYPE_H
#define _1225MP7JRSMEDIARELTIMEPOINTTYPE_H

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOM.hpp>

#include "Mp7JrsDefines.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsMediaRelTimePointTypeFactory.h"
#include "Mp7JrsNode.h"
#include "Mp7JrsConvert.h"


// begin extension included
// file Mp7JrsMediaRelTimePointType_ExtInclude.h

class Mp7JrsmediaTimePointType;
// end extension included


#include "Mp7JrsmediaTimeOffsetType.h"
class IMp7JrsMediaRelTimePointType;
class IMp7JrsxPathRefType;
typedef Mp7JrsPtr< IMp7JrsMediaRelTimePointType> Mp7JrsMediaRelTimePointPtr;
typedef Mp7JrsPtr< IMp7JrsxPathRefType > Mp7JrsxPathRefPtr;

/** 
 * Generated interface IMp7JrsMediaRelTimePointType for class Mp7JrsMediaRelTimePointType<br>
 * Located at: mds-2001.xsd, line 1062<br>
 * Classified: Class<br>
 * Derived from: mediaTimeOffsetType (Pattern)<br>
 */
class MP7JRS_EXPORT IMp7JrsMediaRelTimePointType 
 :
        public IMp7JrsmediaTimeOffsetType
{
public:
    // TODO: make these protected?
    IMp7JrsMediaRelTimePointType();
    virtual ~IMp7JrsMediaRelTimePointType();
    /**
     * Get mediaTimeBase attribute.
     * @return Mp7JrsxPathRefPtr
     */
    virtual Mp7JrsxPathRefPtr GetmediaTimeBase() const = 0;

    /**
     * Get mediaTimeBase attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExistmediaTimeBase() const = 0;

    /**
     * Set mediaTimeBase attribute.
     * @param item const Mp7JrsxPathRefPtr &
     */
    virtual void SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate mediaTimeBase attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidatemediaTimeBase() = 0;

    /**
     * Get OffsetSign element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @return int
     */
    virtual int GetOffsetSign() const = 0;

    /**
     * Set OffsetSign element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @param item int
     */
    // Mandatory			
    virtual void SetOffsetSign(int item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

    /**
     * Get Days element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @return int
     */
    virtual int GetDays() const = 0;

    /**
     * Set Days element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @param item int
     */
    // Mandatory			
    virtual void SetDays(int item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

    /**
     * Get Hours element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @return int
     */
    virtual int GetHours() const = 0;

    /**
     * Set Hours element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @param item int
     */
    // Mandatory			
    virtual void SetHours(int item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

    /**
     * Get Minutes element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @return int
     */
    virtual int GetMinutes() const = 0;

    /**
     * Set Minutes element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @param item int
     */
    // Mandatory			
    virtual void SetMinutes(int item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

    /**
     * Get Seconds element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @return int
     */
    virtual int GetSeconds() const = 0;

    /**
     * Set Seconds element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @param item int
     */
    // Mandatory			
    virtual void SetSeconds(int item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

    /**
     * Get NumberOfFractions element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @return int
     */
    virtual int GetNumberOfFractions() const = 0;

    /**
     * Set NumberOfFractions element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @param item int
     */
    // Mandatory			
    virtual void SetNumberOfFractions(int item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

    /**
     * Get Time_Exist element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @return bool
     */
    virtual bool GetTime_Exist() const = 0;

    /**
     * Set Time_Exist element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @param item bool
     */
    // Mandatory			
    virtual void SetTime_Exist(bool item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

    /**
     * Get FractionsPerSecond element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @return int
     */
    virtual int GetFractionsPerSecond() const = 0;

    /**
     * Set FractionsPerSecond element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @param item int
     */
    // Mandatory			
    virtual void SetFractionsPerSecond(int item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

    /**
     * Gets or creates Mp7JrsNodePtr child elements of MediaRelTimePointType.
     * Currently just supports rudimentary XPath expressions as MediaRelTimePointType contains no child elements of type "Mp7JrsNodePtr" at all.

     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL) = 0;

    
// begin extension included
// file Mp7JrsMediaRelTimePointType_ExtMethodDef.h


public:

	// NOTE: days are ignored in calculations

	/** Get total number of fractions.
	  * @return Get the total number of fractions with respect to the timepoint's number of fractions/second. */
	virtual int GetTotalNrFractions();

	/** Set total number of fractions.
	  * @param fractions Total number of fractions with respect to the timepoint's number of fractions/second. */
	virtual void SetTotalNrFractions(int fractions, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	/** Get absolute value of relative time point using the currently valid mediaTimeBase. 
	  */
	virtual Mp7JrsmediaTimePointPtr GetAbsMediaTimePoint();

	/** Set absolute value of relative time point using the currently valid mediaTimeBase. 
	  */
	virtual void SetAbsMediaTimePoint(Mp7JrsmediaTimePointPtr tp, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);
	
private:

	Mp7JrsmediaTimePointPtr _GetMediaTimeBase();
// end extension included


// no includefile for extension defined 
// file Mp7JrsMediaRelTimePointType_ExtPropDef.h

};




/*
 * Generated class Mp7JrsMediaRelTimePointType<br>
 * Located at: mds-2001.xsd, line 1062<br>
 * Classified: Class<br>
 * Derived from: mediaTimeOffsetType (Pattern)<br>
 * Defined in mds-2001.xsd, line 1062.<br>
 */
class MP7JRS_EXPORT Mp7JrsMediaRelTimePointType :
        public IMp7JrsMediaRelTimePointType
{
    friend class Mp7JrsMediaRelTimePointTypeFactory; // constructs objects

public:
    /*
     * Retrieves the base class of Mp7JrsMediaRelTimePointType
     
     * @return Mp7JrsPtr< Mp7JrsmediaTimeOffsetType >
     */
    virtual Mp7JrsPtr< Mp7JrsmediaTimeOffsetType > GetBase() const;
    /*
     * Get mediaTimeBase attribute.
     * @return Mp7JrsxPathRefPtr
     */
    virtual Mp7JrsxPathRefPtr GetmediaTimeBase() const;

    /*
     * Get mediaTimeBase attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExistmediaTimeBase() const;

    /*
     * Set mediaTimeBase attribute.
     * @param item const Mp7JrsxPathRefPtr &
     */
    virtual void SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate mediaTimeBase attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidatemediaTimeBase();

    /*
     * Get OffsetSign element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @return int
     */
    virtual int GetOffsetSign() const;

    /*
     * Set OffsetSign element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @param item int
     */
    // Mandatory			
    virtual void SetOffsetSign(int item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

    /*
     * Get Days element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @return int
     */
    virtual int GetDays() const;

    /*
     * Set Days element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @param item int
     */
    // Mandatory			
    virtual void SetDays(int item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

    /*
     * Get Hours element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @return int
     */
    virtual int GetHours() const;

    /*
     * Set Hours element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @param item int
     */
    // Mandatory			
    virtual void SetHours(int item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

    /*
     * Get Minutes element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @return int
     */
    virtual int GetMinutes() const;

    /*
     * Set Minutes element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @param item int
     */
    // Mandatory			
    virtual void SetMinutes(int item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

    /*
     * Get Seconds element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @return int
     */
    virtual int GetSeconds() const;

    /*
     * Set Seconds element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @param item int
     */
    // Mandatory			
    virtual void SetSeconds(int item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

    /*
     * Get NumberOfFractions element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @return int
     */
    virtual int GetNumberOfFractions() const;

    /*
     * Set NumberOfFractions element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @param item int
     */
    // Mandatory			
    virtual void SetNumberOfFractions(int item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

    /*
     * Get Time_Exist element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @return bool
     */
    virtual bool GetTime_Exist() const;

    /*
     * Set Time_Exist element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @param item bool
     */
    // Mandatory			
    virtual void SetTime_Exist(bool item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

    /*
     * Get FractionsPerSecond element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @return int
     */
    virtual int GetFractionsPerSecond() const;

    /*
     * Set FractionsPerSecond element.
<br>
     * (Inherited from Mp7JrsmediaTimeOffsetType)
     * @param item int
     */
    // Mandatory			
    virtual void SetFractionsPerSecond(int item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

    /*
     * Gets or creates Mp7JrsNodePtr child elements of MediaRelTimePointType.
     * Currently just supports rudimentary XPath expressions as MediaRelTimePointType contains no child elements of type "Mp7JrsNodePtr" at all.

     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL);

    

    /* Get the class name.
     * @return The name of this class.
     */
    virtual XMLCh * ToText() const;

    /* Parse this type.
     * @param txt buffer to parse from.
     * @return true if parsing was successful, false otherwise.
     */
    virtual bool Parse(const XMLCh * const txt);

    /* Serialialize this object to a DOM tree.
     * @param doc DOM document.
     * @param parent Parent node in DOM tree.
     */
    virtual void Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem = NULL);

    /* Deserialize this object from a DOM tree.
     * @param doc DOM document.
     * @param parent Parent element in DOM tree.
     * @param current Current element in DOM tree.
     */
    virtual bool Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current);

    /* Creates and inserts a Childnode of the specified Type
     * @param type The tag type of the child to be created (tag name).
     * @param xsitype The xsi type of the child to be created if available.
     * @return the created child with the correct dynamic type for the specified type.
     */
    virtual Mp7JrsNodePtr CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype);
         
// no includefile for extension defined 
// file Mp7JrsMediaRelTimePointType_ExtMyMethodDef.h


public:

    virtual ~Mp7JrsMediaRelTimePointType();

protected:
    Mp7JrsMediaRelTimePointType();
    virtual void DeepCopy(const Mp7JrsNodePtr &original);

    virtual Mp7JrsNodeEnum GetAllChildElements() const;

    virtual Mp7JrsNodePtr GetBaseRoot();
    virtual const Mp7JrsNodePtr GetBaseRoot() const;

private:
    Mp7JrsxPathRefPtr m_mediaTimeBase;
    bool m_mediaTimeBase_Exist;

    // Base Pattern
    Mp7JrsPtr< Mp7JrsmediaTimeOffsetType > m_Base;


// no includefile for extension defined 
// file Mp7JrsMediaRelTimePointType_ExtMyPropDef.h


public:
    virtual void Init(); // Called by inheritees to recursively update m_myself
protected:
    void Cleanup();

};

#endif // _1225MP7JRSMEDIARELTIMEPOINTTYPE_H

