/*****************************************************************************/
/* Copyright (C) 2003-2006, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/* This software may only be used under the terms of the JRS MPEG-7 Library  */
/* license, available at                                                     */
/* http://iis.joanneum.at/MPEG-7/JRS-MPEG-7-Lib-License.pdf                  */
/*****************************************************************************/


#ifndef MP7JRSMISSINGELEMENTEXCEPTION_H
#define MP7JRSMISSINGELEMENTEXCEPTION_H

#include "Mp7JrsException.h"

/** Exception class for <code>GetOrCreateFromXPath</code> if the requested index was too high. */
class MP7JRS_EXPORT Mp7JrsMissingElementException : public Mp7JrsException
{
public:
	Mp7JrsMissingElementException(Mp7JrsExSeverity sev, const XMLCh* msg, unsigned int lastvalidindex, Mp7JrsMsgMemory mode = MP7JRS_AUTODELETE) 
		: Mp7JrsException(sev, msg, mode), LastValidIndex(lastvalidindex)
	{ 
	}

	Mp7JrsMissingElementException(Mp7JrsExSeverity sev, const char* msg, unsigned int lastvalidindex, Mp7JrsMsgMemory mode = MP7JRS_AUTODELETE) 
		: Mp7JrsException(sev, msg, mode), LastValidIndex(lastvalidindex)
	{
	}

	// Copy ctor, otherwise we have a memory leak!
	Mp7JrsMissingElementException(const Mp7JrsMissingElementException & other)
		: Mp7JrsException(other), LastValidIndex(other.LastValidIndex)
	{
	}
	
	virtual ~Mp7JrsMissingElementException()
	{
	}
	
	/** Correct index should be the value of GetLastValidIndex() plus 1. */
	unsigned int GetLastValidIndex() { return LastValidIndex; };

private:
	unsigned int LastValidIndex;
};

#endif // MP7JRSMISSINGELEMENTEXCEPTION_H
