/*****************************************************************************/
/* Copyright (C) 2003-2006, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/* This software may only be used under the terms of the JRS MPEG-7 Library  */
/* license, available at                                                     */
/* http://iis.joanneum.at/MPEG-7/JRS-MPEG-7-Lib-License.pdf                  */
/*****************************************************************************/

// ValCollectionType_H

#ifndef _249MP7JRSBEATTYPE_H
#define _249MP7JRSBEATTYPE_H



#include "Mp7JrsDefines.h"
#include "Mp7JrsbeatTypeFactory.h"
#include "Mp7JrsNode.h"
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOM.hpp>
#include "Mp7JrsValueVectorOf.h"

// no includefile for extension defined 
// file Mp7JrsbeatType_ExtInclude.h


class IMp7JrsbeatType;
typedef Mp7JrsPtr< IMp7JrsbeatType> Mp7JrsbeatPtr;


class Mp7JrsbeatType;

/**
 * Generated interface IMp7JrsbeatType for collection Mp7JrsbeatType<br>
 * Located at: audio-2001.xsd, line 794<br>
 * Item type: integer (Value)<br>
 */
class MP7JRS_EXPORT IMp7JrsbeatType :
	public Mp7JrsNode
{
public:

	/** Initializes the collection.
	  * @param maxElems Number of elements for which memory is initially allocated.
	  * @param adoptElems if true, the collection is responsible for deallocating the memory used by its elements. if false, the application has to deallocate the collection elements.
	  */
	virtual void Initialize(unsigned int maxElems) = 0;

	/** Adds a new element to the collection.
	  * @param toAdd Element to be added.
	  */
	virtual void addElement(const int & toAdd, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	/** Sets the element at the given position in the collection.
	  * @param toSet New value of the element.
	  * @param setAt Position of the element which shall be set.
	  */
	virtual void setElementAt(const int & toSet, unsigned int setAt, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	/** Inserts the element at the given position in the collection.
	  * @param toInsert Element to be inserted.
	  * @param insertAt Position where the element will be inserted.
	  */
	virtual void insertElementAt(const int & toInsert, unsigned int insertAt, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	/** Removes all elements from the collection.
	  */
	virtual void removeAllElements(Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	/** Removes the specified element from the collection.
	  * @param removeAt Position of the element to be removed. 
	  */
	virtual void removeElementAt(unsigned int removeAt, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	/** Check whether the collection contains the given element.
	  * @param toCheck Element to be checked for.
	  * @return True if the element is in the collection, false otherwise.
	  */
	virtual bool containsElement(const int &toCheck) = 0;

	/** Get the current capacity of the collection.
	  * @return The number of elements for which memory has been allocated.
	  */
	virtual unsigned int curCapacity() const = 0;

	/** Gets the element at the given position.
	  * @param Position of the element to be retrieved.
	  * @return the given element.
	  */
	virtual const int elementAt(unsigned int getAt) const = 0;

	/** Gets the element at the given position.
	  * @param Position of the element to be retrieved.
	  * @return The given element.
	  */
	virtual int elementAt(unsigned int getAt) = 0;

	/** Get the size of the collection.
	  * @return The current number of elements in the collection.
	  */
	virtual unsigned int size() const = 0;

	/** Allocate additional memory for the speciifed number of elements.
	  * @param length Number of elements to allcate memory for.
	  */
	virtual void ensureExtraCapacity(unsigned int length) = 0;

	// JRS: addition to Xerces collection
	/** Get the position of the specified element. 
	  * @param toCheck Element to be found in the collection.
	  * @return The position of the specified element or -1 if the element has not been found.
	  */
	virtual int elementIndexOf(const int &toCheck) const = 0;

};


class MP7JRS_EXPORT Mp7JrsbeatType :
	public IMp7JrsbeatType
{
	friend class Mp7JrsbeatTypeFactory;

public:
	virtual void Initialize(unsigned int maxElems);

	virtual void addElement(const int & toAdd, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	virtual void setElementAt(const int & toSet, unsigned int setAt, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	virtual void insertElementAt(const int & toInsert, unsigned int insertAt, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	virtual void removeAllElements(Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	virtual void removeElementAt(unsigned int removeAt, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	virtual bool containsElement(const int &toCheck);

	virtual unsigned int curCapacity() const;

	virtual const int elementAt(unsigned int getAt) const;

	virtual int elementAt(unsigned int getAt);

	virtual unsigned int size() const;

	virtual void ensureExtraCapacity(unsigned int length);

	/* Get the underlying array of the collection.
	  */
	virtual const int * rawData() const;

	virtual int elementIndexOf(const int &toCheck) const;

	/* Get the class name.
	 * @return The name of this class.
     */
	virtual XMLCh * ToText() const;

	/* Parse this type.
	 * @param txt buffer to parse from.
	 * @return true if parsing was successful, false otherwise.
	 */
	virtual bool Parse(const XMLCh * const txt);

	/* Serialialize this object to a DOM tree.
	 * @param doc DOM document.
	 * @param parent Parent node in DOM tree.
	 */
	virtual void Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem = NULL);

	/* Deserialize this object from a DOM tree.
	 * @param doc DOM document.
	 * @param parent Parent element in DOM tree.
	 * @param current Current element in DOM tree.
	 */
	virtual bool Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current);

// no includefile for extension defined 
// file Mp7JrsbeatType_ExtMethodDef.h


	virtual ~Mp7JrsbeatType();

protected:
	Mp7JrsbeatType();
	virtual void DeepCopy(const Mp7JrsNodePtr &original);
	virtual Mp7JrsNodeEnum GetAllChildElements() const { return Mp7JrsNodeEnum(); }

private:
	Mp7JrsValueVectorOf <int> * m_Item;

// no includefile for extension defined 
// file Mp7JrsbeatType_ExtPropDef.h


};

#endif // _249MP7JRSBEATTYPE_H
