/*****************************************************************************/
/* Copyright (C) 2003-2006, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/* This software may only be used under the terms of the JRS MPEG-7 Library  */
/* license, available at                                                     */
/* http://iis.joanneum.at/MPEG-7/JRS-MPEG-7-Lib-License.pdf                  */
/*****************************************************************************/


/*! \mainpage The Mp7Jrs library
 *
 * \section intro Introduction
 *
 * Mp7Jrs is a set of C++ classes, implementing an API for the MPEG-7 standard (ISO/IEC V1, 2001). It currently supports parts 3 (Visual), 4 (Audio) and part 5 (Multimedia Description Schemes). 
 *
 * \section about About this API documentation
 *
 * This set of HTML pages is an automatically generated API documentation of the Mp7Jrs classes. It is intended as a reference for a developer using Mp7Jrs. If you are looking for an overview description of the library, instructions on how to setup a project using Mp7Jrs and some commented examples, please look at the document located in the /doc subdirectory of the Mp7Jrs distribution.
 * <br>
 * This API documentation contains the following:
 * <ul>
 * <li>class descriptions, including the extension base type and the schema file and line number where they are defined</li>
 * <li>descriptions of methods, inlcuding a reference to the class from which they are inherited</li>
 * <li>if the type of parameters or return values is abstract, a list of allowed types is provided</li>
 * <li>for collections, a reference to the local type which is the default type of collection elements</li>
 * </ul>
 *
 
 * \section extensions Extensions
 
 			
 * Currently there are no extensions created with this version of Mp7Jrs.
 * All you need to extend types of the core library, is a XML schema,
 * which must be - of course - an extension of the core schema and the extented types
 * must have their own namespace(s). Then an additional library can easily be created
 * along with the core lib.
 *	
 * For more information about this, please contact <a href="http://www.joanneum.at/iis">IIS</a> at JOANNEUM RESEARCH.
 * 
 * \section news Whats new?
 *
 * ATTENTION: A stunning new method for accessing nodes has been introduced.
 * Instead of dealing with difficult intermediate types (and a huge \#include section), it is strongly recommended to use 
 * <code>GetFromXPath()</code> or <code>GetOrCreateFromXPath()</code> methods now.
 * So one single call retrieves a list of desired nodes or creates a complete path with one or more nodes.
 *
 * Example:<br>
 * <code>root.GetFromXPath("A/B/C[last()]/D[3]/E[@xsi:type="Desired"]</code> to get all desired nodes with this type.
 * or:<br>
 * <code>rootnode.GetOrCreateFromXPath("A/B/C[last()]/D[3]/E[@xsi:type="Desired"]</code> creates a node with this type (if there was none before).
 *
 * For a detailed list of supported XPath expressions please read the class documentation.<br>
 * These two methods should be sufficient to handle your requests properly.
 *
 *
 * ATTENTION: The methods <code>Archive::ToWBuffer()</code> and <code>Archive::ToBuffer()</code> are using 
 * Xerces heap now. You *MUST* call the appropriate <code>Archive::DeleteBuffer()</code> methods now
 * , otherwise memory errors will occur! 
 *
 * ATTENTION: The macros for <code>GetObjectType</code> and <code>GetTypeId</code> have been removed,
 * in order to get rid of nasty library problems with existing MS libs! If you need these definitions,
 * redefine them in your source code.
 *
 * ATTENTION: The macro <code>GetClassType</code> has been removed. Use the correspondend <code>GetTypeName()</code>
 * method instead. If you need this definition, please do it in your source code.
 *
 * ATTENTION: The method <code>GetNextSibling()</code> was renamed to <code>GetNextAkin()</code> because
 * of interferences with a macro from MS XML library...
 * If you need this definition, please do it in your source code.
 *
 * Other changes:
 * <ul>
 * <li>Subelements of type string are now created automatically (as empty string) if they are mandatory.
 * <li>Abstract classes automatically handle the <code>UseTypeAttribute</code> flag, needed for correct serialisation.</li>
 * <li>Interface setter methods additionally create VS7 IntelliSense comments for mandatory elements.</li>
 * <li>Library version string can be queried with <code>GetVersion()</code> method.</li>
 * </ul>
 
 */
